/*
 * Echo Server Example: Main Module.
 */
#include <sys/types.h>
#include <arpa/inet.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Exported by the Server Module (../server/echo.c) */
extern void start_server(uint16_t, int);

#define DEFAULT_PORT		1234
#define DEFAULT_CONNECTIONS	3

static uint16_t listen_port = DEFAULT_PORT;
static int max_connections = DEFAULT_CONNECTIONS;


static void
usage(const char *prog)
{
	fprintf(stderr, "usage: %s <options>\n"
	    "where <options> are:\n"
	    "    -h        This help.\n"
	    "    -p <port> Listening port (default is %d)\n"
	    "    -m <max>  Max number of concurrent connections "
	    "(default is %d)\n\n",
	    prog, DEFAULT_PORT, DEFAULT_CONNECTIONS);

	exit(-1);
}


static void
parse_options(int argc, char **argv)
{
	char *e;
	int i;

	for (i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-h")) {
			usage(argv[0]);
			/* NOTREACHED */
		}

		if (i + 1 >= argc) {
			fprintf(stderr, "%s: Invalid number of arguments\n",
			    argv[0]);
			exit(-1);
			/* NOTREACHED */

		} else if (!strcmp(argv[i], "-p")) {
			/*
			 * Set listening port.
			 */
			listen_port = (uint16_t)strtol(argv[++i], &e, 0);
			if (*e != '\0') {
				fprintf(stderr, "%s: Invalid -p argument: %s\n",
				    argv[0], argv[i]);
				exit(-1);
				/* NOTREACHED */
			}

		} else if (!strcmp(argv[i], "-m")) {
			/*
			 * Set max number of concurrent connections.
			 */
			max_connections = (int)strtol(argv[++i], &e, 0);
			if (*e != '\0') {
				fprintf(stderr, "%s: Invalid -m argument: %s\n",
				    argv[0], argv[i]);
				exit(-1);
				/* NOTREACHED */
			}

		} else {
			fprintf(stderr, "%s: Invalid command line option: %s\n",
			    argv[0], argv[i]);
			exit(-1);
			/* NOTREACHED */
		}
	}
}


int
main(int argc, char **argv)
{
	parse_options(argc, argv);

	printf("Echo Server started\n"
	    "  Listening port: %d\n"
	    "  Max concurrent connections: %d\n\n",
	    listen_port,
	    max_connections);

	start_server(listen_port, max_connections);

	return 0;
}


