#
# Variables to set before including this file:
#
#	PLPREFIX=
#		Installation Base Directory;
#		MANDATORY if PLBASE is not defined
#
#	PLBASE=	Base Directory;
# 		MANDATORY if PLPREFIX is not defined
#
#	PLOS=	Target Operating System (lowercase);
# 		MANDATORY
#
#	PLARCH=	Target Architecture (Processor type) (lowercase);
# 		MANDATORY
#
#	PLEXTERNAL=
#		This variable should be defined when the code
# 		to build is not part of Peludo distribution;
# 		see pbase.mk.
#
#	PLOPT=<yes/no>
#		The target should be compiled with compiler
#		optimizations enabled; default is "no".
#
#	PLDEBUG=<yes/no>
#		The target should be compiled in debug mode;
#		default is "no".
#
# Automatically generated variables available to makefiles:
#
#	PLTARGET=
#		<os>_<arch>
#
# If debug mode is active (PLDEBUG=yes) then:
#	PLTARGET=
#		<os>_<arch>_debug
#
# Automatic C macros:
#
#	__PELUDO__
#
#	PLOS_<os>
#		Target Operating System;
#
#	PLARCH_<arch>
#		Target Architecture;
#
#	PL_<os>_<arch>
#		Target Platform (OS/ARCH);
#
#	PLSTATIC
#		Target is a statically linked executable;
#
#	PLDEBUG	Target is compiled in debug mode.
#
#
ifndef PLBASE
ifndef PLPREFIX
PLERROR+=		"PLBASE or PLPREFIX must be defined."
endif
endif

ifndef PLOS
PLERROR+=		"PLOS not defined."
endif

ifndef PLARCH
PLERROR+=		"PLARCH not defined."
endif

ifdef PLERROR
.PHONY:	all error
all: error

error:
	@printf "\nERROR: %s\n" ${PLERROR} >&2
	@exit 1

else


PLPREFIX?=		${abspath ${PLBASE}/../../}/build
PLBASE?=		${abspath ${PLPREFIX}}/${PLOS}_${PLARCH}
_PLPREFIX=		${abspath ${PLPREFIX}}
_PLBASE=		${abspath ${PLBASE}}


#
# Default hosting platform
#
PLHOST_OS:=		${shell uname -s | tr [A-Z] [a-z]}
PLHOST_ARCH:=		${shell uname -m | sed -e 's,amd64,i386,' -e 's,x86_64,i386,' -e 's,.*86,i386,'}
PLHOST_REALARCH:=	${shell uname -m}
PLHOST:=		${PLHOST_OS}_${PLHOST_ARCH}


#
# Autogenerated variables
#
ifneq (${PLDEBUG}, yes)
PLTARGET:=		${PLOS}_${PLARCH}
else
PLTARGET:=		${PLOS}_${PLARCH}_debug
MAKEARGS+=		PLDEBUG=yes
endif


#
# Binaries
#
TOOLCHAINDIR?=		${_PLPREFIX}/${PLHOST_OS}_${PLHOST_ARCH}
TOOLCHAIN_BINDIR=	${TOOLCHAINDIR}/bin
TOOLCHAIN_LIBDIR=	${TOOLCHAINDIR}/lib

override AR=		${TOOLCHAIN_BINDIR}/ar
override CC=		${TOOLCHAIN_BINDIR}/gcc
override CPP=		${TOOLCHAIN_BINDIR}/cpp
override LD=		${TOOLCHAIN_BINDIR}/ld
override NM=		${TOOLCHAIN_BINDIR}/nm
override OBJCOPY=	${TOOLCHAIN_BINDIR}/objcopy
override OBJDUMP=	${TOOLCHAIN_BINDIR}/objdump
override RANLIB=	${TOOLCHAIN_BINDIR}/ranlib
override STRIP=		${TOOLCHAIN_BINDIR}/strip

AWK?=			awk
CP?=			cp
CHMOD?=			chmod
DD?=			dd
FIND?=			find
INSTALL?=		install
LN?=			ln
OPENSSL?=		openssl
MV?=			mv
MKDIR?=			mkdir
PATCH?=			patch
SED?=			sed
TOUCH?=			touch

ifeq (${PLHOST_OS}, linux)
PATCH_FLAGS=		-b
else
PATCH_FLAGS=		-u
endif


#
# Compilation flags
#
ARFLAGS?=		ru

PLHOST_STATIC_CFLAGS+=	-pipe \
			-Wall \
			-Werror \
			-fno-stack-protector \
			-fno-strict-aliasing \
			-fno-builtin \
			-march=${PLHOST_ARCH}

STATIC_CFLAGS+=		-pipe \
			-Wall \
			-Werror \
			-fno-stack-protector \
			-fno-strict-aliasing \
			-fno-builtin \
			-march=${PLARCH}

CFLAGS?=		-pipe \
			-Wall \
			-Werror \
			-Wno-main \
			-fPIC \
			-fno-builtin \
			-fno-zero-initialized-in-bss \
			-fno-stack-protector \
			-fno-strict-aliasing \
			-march=${PLARCH}

# 64 bits host building a 32 bits target
ifeq (${PLHOST_REALARCH}, x86_64)
PLHOST_STATIC_CFLAGS+=	-m32
PLHOST_STATIC_LDFLAGS+=	-melf_i386
STATIC_CFLAGS+=		-m32
STATIC_LDFLAGS+=	-melf_i386
CFLAGS+=		-m32
LDFLAGS+=		-melf_i386
else ifeq (${PLHOST_REALARCH}, amd64)
PLHOST_STATIC_CFLAGS+=	-m32
PLHOST_STATIC_LDFLAGS+=	-melf_i386_fbsd
STATIC_CFLAGS+=		-m32
STATIC_LDFLAGS+=	-melf_i386_fbsd
CFLAGS+=		-m32
LDFLAGS+=		-melf_i386_fbsd
endif

ifndef PLEXTERNAL
CFLAGS+=		-fvisibility=hidden
endif

ifeq (${PLDEBUG}, yes)
PLHOST_STATIC_CFLAGS+=	-g -O0
PLHOST_STATIC_DEFS+=	-DPLDEBUG
STATIC_CFLAGS+=		-g -O0
STATIC_DEFS+=		-DPLDEBUG
CFLAGS+=		-g -O0
DEFS+=			-DPLDEBUG
else

ifneq (${PLOPT}, yes)
PLHOST_STATIC_CLFAGS+=	-O0
STATIC_CFLAGS+=		-O0
CFLAGS+=		-O0
else
PLHOST_STATIC_CLFAGS+=	-O2
STATIC_CFLAGS+=		-O2
CFLAGS+=		-O2
endif
endif

PLHOST_STATIC_CPPFLAGS+=-nostdinc
STATIC_CPPFLAGS+=	-nostdinc
CPPFLAGS+=		-nostdinc

ifndef PLEXTERNAL
PLHOST_STATIC_CPPFLAGS+=-I${_PLBASE}/src/include \
			-I${_PLBASE}/src/include/std

STATIC_CPPFLAGS+=	-I${_PLBASE}/src/include \
			-I${_PLBASE}/src/include/std

CPPFLAGS+=		-I${_PLBASE}/src/include \
			-I${_PLBASE}/src/include/std
else
PLHOST_STATIC_CPPFLAGS+=-I${_PLPREFIX}/include \
			-I${_PLPREFIX}/include/std

STATIC_CPPFLAGS+=	-I${_PLPREFIX}/include \
			-I${_PLPREFIX}/include/std

CPPFLAGS+=		-I${_PLPREFIX}/include \
			-I${_PLPREFIX}/include/std
endif

PLHOST_STATIC_LDFLAGS+=	-static \
			-nostdlib

STATIC_LDFLAGS+=	-static \
			-nostdlib

ifndef PLEXTERNAL
LDFLAGS+=		-L${_PLBASE}/mk \
			-L${_PLBASE}/src/lib \
			-L${_PLBASE}/src/core/kernel
STATIC_CRT0DIR=		${_PLBASE}/src/lib
CRT0DIR=		${_PLBASE}/src/lib
else
LDFLAGS+=		-L${_PLPREFIX}/mk \
			-L${_PLBASE}/lib
STATIC_CRT0DIR=		${_PLBASE}/lib
CRT0DIR=		${_PLBASE}/lib
endif

PLHOST_STATIC_CRT0DIR=	${TOOLCHAIN_LIBDIR}

LDFLAGS+=		-shared \
			-nostdlib \
			--no-omagic \
			-z nodefaultlib \
			-Tscript.lds

# XXX
#ifeq (${PLOS}, win32)
#CPPFLAGS+=		-I${_PLBASE}/src/lib/win32/include
#LDFLAGS+=		-L${_PLBASE}/src/lib/win32/lib
#endif

PLHOST_STATIC_DEFS+=	-D__PELUDO__ \
			-DPLSTATIC \
			-DPLOS_${PLHOST_OS} \
			-DPLARCH_${PLHOST_ARCH} \
			-DPL_${PLHOST_OS}_${PLHOST_ARCH}

STATIC_DEFS+=		-D__PELUDO__ \
			-DPLSTATIC \
			-DPLOS_${PLOS} \
			-DPLARCH_${PLARCH} \
			-DPL_${PLOS}_${PLARCH}

DEFS+=			-D__PELUDO__ \
			-DPLOS_${PLOS} \
			-DPLARCH_${PLARCH} \
			-DPL_${PLOS}_${PLARCH}

PLHOST_STATIC_CRT0=	${PLHOST_STATIC_CRT0DIR}/crt0_${PLHOST}.o
STATIC_CRT0=		${STATIC_CRT0DIR}/crt0_${PLTARGET}.o
CRT0=			${CRT0DIR}/plcrt0_${PLTARGET}.o


#
# Internal Binaries.
#
DATA2PLD?=		${TOOLCHAIN_BINDIR}/data2pld
DIR2PLD?=		${TOOLCHAIN_BINDIR}/dir2pld
ELF2PLD?=		${TOOLCHAIN_BINDIR}/elf2pld
PLB2C?=			${TOOLCHAIN_BINDIR}/plb2c
PLB2ELF?=		${TOOLCHAIN_BINDIR}/plb2elf
PLDHASH?=		${TOOLCHAIN_BINDIR}/pldhash
PLZMA?=			${TOOLCHAIN_BINDIR}/plzma
PRUN?=			${TOOLCHAIN_BINDIR}/prun

DATA2PLD_FLAGS+=	-v \
			-H ${PLDHASH} \
			-Z "${PLZMA} e"

DIR2PLD_FLAGS+=		-v \
			-H ${PLDHASH}

ELF2PLD_FLAGS+=		-v \
			-H ${PLDHASH}

PLB2ELF_FLAGS+=		-p ${PLOS}_${PLARCH}

ifeq (${PLDEBUG}, yes)
DATA2PLD_FLAGS+=	--hashsection
DIR2PLD_FLAGS+=         --hashsection
ELF2PLD_FLAGS+=         --hashsection
endif

# XXX
#ifeq (${PLOS}, win32)
#ELF2PLD_FLAGS+=         --native
#endif


#
# Installation Flags.
#
OWNER_DATA?=		`id -u`
GROUP_DATA?=		`id -g`
MODE_DATA?=		644

OWNER_BIN?=		`id -u`
GROUP_BIN?=		`id -g`
MODE_BIN?=		755

OWNER_DIR?=		`id -u`
GROUP_DIR?=		`id -g`
MODE_DIR?=		755


#
# Installation Directories.
#
BINDIR?=		${_PLPREFIX}/${PLOS}_${PLARCH}/bin
DOCDIR?=		${_PLPREFIX}/doc
INCDIR?=		${_PLPREFIX}/include
LIBDIR?=		${_PLPREFIX}/${PLOS}_${PLARCH}/lib
MKFDIR?=		${_PLPREFIX}/mk
MODDIR?=		${_PLPREFIX}/${PLOS}_${PLARCH}/lib


endif
