#
# plbtarget.mk is used to generate PLB files.
#
# PLB=		Name of the .plb file to generate.
#		MANDATORY variable.
#
# PLBELF=	If this variable is set an ELF is also generated
#		embedding the PLB.
#
# MODS=		List of modules to include into the PLB file
#		(in dependency order).
# 		MANDATORY variable.
#
# PLBOOT=	Bootstrap code.
#		(Default is ${LIBDIR}/krn0_${PLTARGET}.plo)
#
# PLKERNEL=	Kernel code.
#		(Default is ${LIBDIR}/krn1_${PLTARGET}.plo)
#
# LIBC=		C runtime.
#		(Default is ${LIBDIR}/libplc_${PLTARGET}.pld)
#
# LIBPTHRAEDS=	Multithread library.
#		(Default is ${LIBDIR}/libpthrad_${PLTARGET}.pld)
#
ifndef PLB
PLERROR+=	"PLB not defined."
endif

ifndef MODS
PLERROR+=	"MODS not defined."
endif

PLBOOT?=	${LIBDIR}/krn0_${PLTARGET}.plo
PLKERNEL?=	${LIBDIR}/krn1_${PLTARGET}.plo
PLENDM?=	${LIBDIR}/end_${PLTARGET}.pld

LIBC?=		${LIBDIR}/libplc_${PLTARGET}.pld
LIBPTHREADS?=	${LIBDIR}/libpthread_${PLTARGET}.pld

.PHONY: plb plbelf

ifndef PLERROR
CLEANFILES+=	${PLB}

plb: ${PLB}

${PLB}: ${PLBOOT} ${PLKERNEL} ${MODS} ${PLENDM}
	cat $^ > $@

ifdef PLBELF
CLEANFILES+=	${PLBELF}
${PLBELF}: ${PLB}
	${PLB2ELF} -p ${PLTARGET} $< $@

endif

else
plb plbelf:

endif

