
SECTIONS {
	/*
	 * Because .text references .data using fixed values (no relocations)
	 * the PLD loader must create these sections at once (only one mmap
	 * for both) at the same time it is needed a page alignment between
	 * them because of memory permissions that are not the same (X vs. RW).
	 * Notice that the PLD file will not be aligned, the loader aligns.
	 *
	 * FIXME: This can be solved avoiding the REL32 relocation type.
	 */
	. = ALIGN(0x1000);
	.text : { *(.text* .init* .fini* .ctor* .dtor* .plt* .gnu.linkonce.*) }

	/*
	 * .text and .data should be page aligned.
	 */
	. = ALIGN(0x1000);

	/*
	 * GCC uses 2 internal sections:
	 *  .eh_frame (Exception Handler)
	 *  .jcr (Java Class Registration)  
	 * Without them libgcc::_init segfaults even in pure C code.
	 */
	.data : { *(.data* .rodata* .got* .jcr .eh_frame_hdr .eh_frame) }

	/* 
	 * .data and .bss should be 4 bytes aligned.
	 *
	 * FIXME: What about 64 bits?
	 */
	. = ALIGN(0x0004);
	.bss : { *(.bss) }

	/*
	 * Other (unused) elf sections go here.
	 */
	. = ALIGN(0x1000);
	.dynamic : {}
	.dynsym : {}
	.dynstr : {}
	.gnu.hash : {}
	.hash : {}
	.rel.dyn : {}
}

