/**
 * @file src/include/plarchdef.h
 * Architecture only Dependent Definitions.
 */
#ifndef PLARCHDEF_H
#define PLARCHDEF_H


/*************
 * i386-32
 *************/
#if defined(PLARCH_i386)

typedef unsigned char		uint8;
typedef unsigned short		uint16;
typedef unsigned int		uint32;
typedef unsigned long long	uint64;

typedef signed char		int8;
typedef signed short		int16;
typedef signed int		int32;
typedef signed long long	int64;

#define PLPAGE_SIZE		0x1000

#define	PLLITTLE_ENDIAN
#undef	PLBIG_ENDIAN

#define	PLSTACK_DIRECTION_DOWN
#undef	PLSTACK_DIRECTION_UP

typedef uint16			PLFPU_CONTROL;

#define PLFPU_CONTROL_SET(x)	do {					\
	__asm__ __volatile__("fldcw %0" :: "m" (x));			\
} while (0)

#define PLFPU_CONTROL_GET(x)	do {					\
	__asm__ __volatile__("fnstcw %0" : "=m" (x));			\
} while (0)

#define PLFPU_EXTENDED		0x0300
#define PLFPU_DOUBLE		0x0200
#define PLFPU_SINGLE		0x0000


/*************
 * AMD64
 *************/
#elif defined(PLARCH_x86_64)

typedef unsigned char		uint8;
typedef unsigned short		uint16;
typedef unsigned int		uint32;
typedef unsigned long 		uint64;

typedef signed char		int8;
typedef signed short		int16;
typedef signed int		int32;
typedef signed long 		int64;

#define PLPAGE_SIZE		0x1000

#define	PLLITTLE_ENDIAN
#undef	PLBIG_ENDIAN

#define	PLSTACK_DIRECTION_DOWN
#undef	PLSTACK_DIRECTION_UP

/* XXX */
#error "PLFPU_xxx and PLALLREG_STORAGE not defined on x86_64"


#else
#error "UNKNOWN ARCHITECTURE"
#endif	/* PLARCH_i386 */


/*******************
 * ARCH INDEPENDENT
 *******************/
typedef	unsigned long		size_t;
typedef	signed long		ssize_t;
typedef	signed long int		intptr_t;
typedef	unsigned long int	uintptr_t;
typedef long			ptrdiff_t;
typedef char *			caddr_t;

#if defined(PLLITTLE_ENDIAN)

#define PLN32H(x)		((((uint32)(x)) << 24)			| \
				((((uint32)(x)) << 8) & 0x00FF0000)	| \
				((((uint32)(x)) >> 8) & 0x0000FF00)	| \
				(((uint32)(x)) >> 24))

#define PLN16H(x)		((((uint16)(x)) << 8) | (((uint16)(x)) >> 8))

#else

#define PLN32H(x)
#define PLN16H(x)

#endif	/* !PLLITTLE_ENDIAN */

#define PLPAGE_MASK		(PLPAGE_SIZE - 1)

#define PLALIGN(x, a)		(((x) + ((a) - 1)) & ~((a) - 1))

#define PLIS_ALIGNED(x, a)	((x) == ((x) & ((a) - 1)))

#define PLIS_PALIGNED(x)	PLIS_ALIGNED(x, PLPAGE_SIZE)

#define PLNULL			((void *)0L)

#ifndef NULL
#define NULL			PLNULL
#endif

#define PLEXPORT		__attribute__((visibility("default")))

/*
 * float.h - libgcc (?)
 * XXX
 */
#define DBL_DIG			15
#define FLT_ROUNDS		1
#define DBL_MAX_EXP		1024
#define DBL_MAX_10_EXP		308
#define FLT_RADIX		2


#endif	/* PLARCHDEF_H */
