/**
 * @file src/include/plist.h
 * Peludo (circular) Lists.
 */
#ifndef PLIST_H
#define PLIST_H

#include <plarchdef.h>

typedef struct _PLIST PLIST;

struct _PLIST {
	PLIST	*next;
	PLIST	*prev;
};

#define PLIST_INITIALIZED	{ NULL, NULL }

#define PLIST_INIT(root)	((root)->next = (root)->prev = (PLIST *)(root))

#define	plist_empty(root)	((root)->next == (root))

#define PLIST_FOREACH(root, var, member)				\
		for ((var) = (__typeof__(var))((root)->next);		\
		     (var) != (__typeof__(var))(root);			\
		     (var) = (__typeof__(var))((var)->member.next))

#define PLIST_FOREACH_REV(root, var, member)				\
		for ((var) = (__typeof__(var))((root)->prev);		\
		     (var) != (__typeof__(var))(root);			\
		     (var) = (__typeof__(var))((var)->member.prev))

#define PLIST_FIRST(root, type)	((type)((root)->next))


/**
 * Add a node into a list.
 * @see PLIST
 */
void PLEXPORT plist_add(PLIST *node, PLIST *next, PLIST *prev);


/**
 * Remove a node from a list.
 */
void PLEXPORT plist_del(PLIST *node);


/**
 * Add a node at the end of a list (queue).
 * @see PLIST
 * @see plist_add()
 */
static inline void plist_add_tail(PLIST *root, PLIST *node)
{
        plist_add(node, root, root->prev);
}


/**
 * Add a node at the beginning of a list (stack).
 * @see PLIST
 * @see plist_add()
 */
static inline void plist_add_head(PLIST *root, PLIST *node)
{
	plist_add(node, root->next, root);
}


#endif	/* PLIST_H */
