/**
 * @file include/std/stdio.h
 * Peludo LibC - stdio.h
 */
#ifndef __STDIO_H__
#define __STDIO_H__

#include <plarchdef.h>
#include <stdarg.h>
#include <limits.h>

#define BUFSIZ		1024

#define EOF		(-1)

#define FILENAME_MAX	PATH_MAX

#define MAXPATHLEN	PATH_MAX	/* XXX sys/param.h? */

typedef	struct _FILE {
	long	f_fd;
} FILE;

extern FILE *stdin;
extern FILE *stdout;
extern FILE *stderr;

FILE	*fopen(const char *, const char *);
int	 fclose(FILE *);
int	 fflush(FILE *);
int	 fileno(FILE *);
size_t	 fread(void *, size_t, size_t, FILE *);
size_t	 fwrite(const void *, size_t, size_t, FILE *);

void	 perror(const char *);

/* XXX
 * plprint() family does not support all printf() format options.
 * See src/lib/plc/plvnprint_cb.c
 */

int	 fprintf(FILE *, const char *, ...);
int	 vfprintf(FILE *, const char *, va_list);

#define	printf(args...)		fprintf(stdout, args)
#define vprintf(args...)	vfprintf(stdout, args)

#define sprintf			plsprint
#define vsprintf		plvsprint

#define snprintf		plsnprint
#define vsnprintf		plvsnprint

int	 remove(const char *);
int	 rename(const char *, const char *);


#endif	/* __STDIO_H__ */
