/**
 * Peludo libc.
 */
#ifndef _UNISTD_H_
#define _UNISTD_H_

#ifdef PLOS_win32
/* XXX */
#error "unistd.h NOT PORTED TO WIN32."
#endif

#include <plc.h>

#define STDIN_FILENO		0
#define STDOUT_FILENO		1
#define STDERR_FILENO		2


int	 close(int);

//XXX int	 chdir(const char *) __attribute__((weak, alias("plsys_chdir")));
#define	 chdir(path)	plsys_chdir((path));

int	 dup(int);
int	 dup2(int, int);
char	*getcwd(char *, size_t);
int	 fdatasync(int);
int	 fsync(int);
ssize_t	 pread(int, void *, size_t, off_t);
ssize_t	 pwrite(int, const void *, size_t, off_t);
ssize_t	 read(int, void *, size_t);
ssize_t	 write(int, const void *, size_t);

pid_t	 fork();

/* Not standard. But defined in linux and solaris. Not in unistd.h though */
ssize_t	 sendfile(int, int, off_t *, size_t);


static inline int __attribute__((always_inline))
access(const char *path, int mode)
{
	return ((plsys_access(path, mode) == PLSTATUS_SUCCESS) ? 0 : -1);
}


static inline int __attribute__((always_inline))
execve(const char *path, char *const argv[], char *const envp[])
{
	return plsys_execve(path, argv, envp);
}


static inline int __attribute__((always_inline))
ftruncate(int fd, off_t len)
{
	return ((plsys_ftruncate((long)fd, (long)len) == PLSTATUS_SUCCESS)
	    ? 0 : -1);
}


static inline int __attribute__((always_inline))
gethostname(char *name, size_t len)
{
	return ((plsys_gethostname(name, len) == PLSTATUS_SUCCESS) ? 0 : -1);
}


static inline int __attribute__((always_inline))
getpagesize()
{
	return PLPAGE_SIZE;
}


static inline pid_t __attribute__((always_inline))
getpid()
{
	return ((pid_t)plsys_getpid());
}


static inline off_t __attribute__((always_inline))
lseek(int fd, off_t offset, int whence)
{
	return ((plsys_lseek((long *)&offset, (long)fd, (long)offset, whence)
	    == PLSTATUS_SUCCESS) ? offset : -1);
}


static inline int __attribute__((always_inline))
pipe(int filedes[2])
{
	return ((plsys_pipe((long *)filedes) == PLSTATUS_SUCCESS) ? 0 : -1);
}


static inline ssize_t __attribute__((always_inline))
readlink(const char *path, char *buf, size_t bufsiz)
{
	long siz;

	siz = (long)bufsiz;
	return ((plsys_readlink(path, buf, &siz) == PLSTATUS_SUCCESS) ?
	     (ssize_t)siz : -1);
}


static inline int __attribute__((always_inline))
rmdir(const char *path)
{
	return ((plsys_rmdir(path) == PLSTATUS_SUCCESS) ? 0 : -1);
}


static inline pid_t __attribute__((always_inline))
setsid()
{
	return ((pid_t)plsys_setsid());
}


static inline int __attribute__((always_inline))
unlink(const char *path)
{
	return ((plsys_unlink(path) == PLSTATUS_SUCCESS) ? 0 : -1);
}


#endif	/* _UNISTD_H_ */
