/**
 * @file src/lib/plc/pllog.c
 */
#ifdef PLDEBUG

#include <plc.h>
#include <pllog.h>
#include <plspinlock.h>


static PLOCK _debuglock = PLOCK_INITIALIZED;


/**
 * Log a debug message on a multithreaded environment.
 * In release mode this function is an empty macro.
 *
 * @param fmt (IN) Format string (in <code>plprint()</code> format)
 * specifying how to process the following arguments;
 *
 * @param va (IN) List of arguments.
 *
 * @see plva_list
 * @see plprint()
 */
void PLEXPORT
plvdebug(const char *fmt, plva_list va)
{
	sigset_t cset;

	plsigblock(&cset);
	plock_splock(&_debuglock);

	plprint("%d/%d: ", plsys_getpid(), plsys_gettid());
	plvprint(fmt, va);

	plock_unlock(&_debuglock);
	plsigunblock(&cset);
}


/**
 * Log a debug message on a multithreaded environment.
 * In release mode this function is an empty macro.
 *
 * @param fmt (IN) Format string (in <code>plprint()</code> format)
 * specifying how to process the following arguments (if any).
 *
 * @see plvdebug()
 */
void PLEXPORT
pldebug(const char *fmt, ...)
{
	plva_list va;

	plva_start(va, fmt);
	plvdebug(fmt, va);
	plva_end(va);
}


#endif	/* PLDEBUG */

