/**
 * @file src/lib/plc/std/stdio_fopen.c
 */
#include <plc.h>
#include <stdio.h>
#include <string.h>


struct _mode2flags {
	char	*mode;
	int	 flags;
	int	 cmode;
};


static struct _mode2flags *
_parsemode(const char *mode)
{
	/* Dont care about "b" */
	static struct _mode2flags mode2flags[] = {
		{ "r",	O_RDONLY,	0			},
		{ "r+", O_RDWR,		0			},
		{ "w",	O_WRONLY,	O_CREAT | O_TRUNC	},
		{ "w+",	O_RDWR,		O_CREAT | O_TRUNC	},
		{ "a",	O_WRONLY,	O_CREAT | O_APPEND	},
		{ "a+",	O_RDWR,		O_CREAT | O_APPEND	},
		{ NULL, 0, 0 }
	};
	struct _mode2flags *p;

	for (p = mode2flags; p->mode != NULL; p++) {
		if (!strcmp(mode, p->mode)) {
			return p;
		}
	}

	return NULL;
}


FILE PLEXPORT *
fopen(const char *path, const char *mode)
{
	struct _mode2flags *of;
	FILE *f;
	long fd;

	if (path == NULL || mode == NULL || (of = _parsemode(mode)) == NULL) {
		errno = EINVAL;
		return NULL;
	}

	if (plsys_open(&fd, path, of->flags, of->cmode) != PLSTATUS_SUCCESS) {
		return NULL;
	}

	if ((f = plmalloc(sizeof(FILE))) == NULL) {
		plsys_close(fd);
		return NULL;
	}

	plbzero((uint8 *)f, sizeof(FILE));
	f->f_fd = fd;

	return f;
}


