/**
 * @file src/lib/plc/std/sys_mman_mmap.c
 */
#include <plc.h>
#include <sys/mman.h>


static PLMFLAGS
unix2pl(int prot)
{
	PLMFLAGS plf;

	plf = 0;

	if (prot & PROT_EXEC) {
		plf |= PLMFLAGS_X;
	}

	if (prot & PROT_READ) {
		plf |= PLMFLAGS_R;
	}

	if (prot & PROT_WRITE) {
		plf |= PLMFLAGS_W;
	}

	return plf;
}


void PLEXPORT *
mmap(void *start, size_t len, int prot, int flags, int fd, off_t offset)
{
	PLSTATUS st;
	long addr;

	if ((flags & MAP_ANONYMOUS) == MAP_ANONYMOUS || fd < 0) {
		/*
		 * Take memory from the system.
		 */
		st = plsys_palloc(&addr, len, unix2pl(prot));
	} else {
		/*
		 * Mmap a file.
		 */
		st = plsys_mmapfile(&addr, (long)fd, len, flags, (long)prot,
		    (long)offset);

	}

	return ((st == PLSTATUS_SUCCESS) ? (void *)addr : MAP_FAILED);
}


int PLEXPORT
munmap(void *start, size_t len)
{
	return ((plsys_munmap((long)start, len) == PLSTATUS_SUCCESS) ? 0 : -1);
}


