
#include <unistd.h>


/**
 * Force all currently queued I/O operations associated with the file indicated
 * by file descriptor fildes to the synchronised I/O completion state.
 * <b>This function is a cancellation point.<b>
 *
 * @param fd (IN) File descriptor.
 *
 * @return On success <code>0</code> is returned; otherwise, <code>-1</code>
 * and <code>errno</code> is set properly.
 *
 * @see http://opengroup.org/onlinepubs/007908775/xsh/fdatasync.html
 *
 * @see plsys_fdatasync()
 */
int PLEXPORT
fdatasync(int fd)
{
	PLSTATUS st;

	plenter_cancel_point();

	st = plsys_fdatasync((long)fd);

	plleave_cancel_point();

	return ((st == PLSTATUS_SUCCESS) ? 0 : -1);
}


