/**
 * @file src/lib/plc/sysdep/linux/i386/plsys_opendir.c
 */
#include <plc.h>


struct _plsys_opendir_handle {
	long	fd;
	long	count;		/* Number of bytes retrieved by user */
	long	size;		/* Number of bytes in the chunk */

	/* Just make the whole structure page sized */
	uint8	buf[PLPAGE_SIZE - (sizeof(long) * 3)];
};


PLSTATUS PLEXPORT
plsys_opendir(long *handle, const char *path)
{
	struct _plsys_opendir_handle *h;
	PLSTATUS st;
	long fd;

	if (handle == NULL || path == NULL) {
		return PLSTATUS_INVALID;
	}

	/*
	 * Open the directory file.
	 */
	st = plsys_open(&fd, path, O_RDONLY | O_DIRECTORY, 0);
	if (st != PLSTATUS_SUCCESS) {
		return st;
	}

	/*
	 * Create the buffer for retrievings using plsys_readdir().
	 * XXX maybe malloc is better
	 */
	st = plsys_palloc((long *)&h, sizeof(struct _plsys_opendir_handle),
	    PLMFLAGS_R | PLMFLAGS_W);
	if (st != PLSTATUS_SUCCESS) {
		plsys_close(fd);
		return st;
	}

	h->fd = fd;
	h->count = 0;
	h->size = 0;
	*handle = (long)h;

	return PLSTATUS_SUCCESS;
}


void PLEXPORT
plsys_closedir(long handle)
{
	if (handle != 0) {
		plsys_close(((struct _plsys_opendir_handle *)handle)->fd);
		plsys_pfree(handle, sizeof(struct _plsys_opendir_handle));
	}
}


PLSTATUS PLEXPORT
plsys_readdir(long handle, struct dirent **de)
{
	struct _plsys_opendir_handle *h;
	struct linux_dirent *lde;
	long size;

	if (handle == 0 || de == NULL)  {
		return PLSTATUS_INVALID;
	}

	h = (struct _plsys_opendir_handle *)handle;

	/*
	 * Read a chunk of the directory content.
	 */
	if (h->count >= h->size) {
		size = plsyscall(SYS_getdents, h->fd, h->buf, sizeof(h->buf));
		if (size < 0) {
			return PLSTATUS_FAILED;
		}

		if (size == 0) {
			/*
			 * No more info to retrieve.
			 */
			return PLSTATUS_EOF;
		}

		h->count = 0;
		h->size = size;
	}

	/*
	 * Get the next directory entry.
	 */
	lde = (struct linux_dirent *)(h->buf + h->count);
	*de = lde;

	h->count += lde->d_reclen;

	return PLSTATUS_SUCCESS;
}


