/**
 * @file src/lib/plc/sysdep/linux/i386/plsys_tsigmask.c
 */
#include <plc.h>


PLSTATUS PLEXPORT
plsys_tsigmask_nt(int how, const sigset_t *newset, sigset_t *oldset)
{
	if (plsyscall(SYS_rt_sigprocmask, how, newset, oldset, NSIG / 8, 0, 0)
	    != -1) {
		return PLSTATUS_SUCCESS;
	}

	return ((errno == EAGAIN) ? PLSTATUS_TRY_AGAIN : PLSTATUS_FAILED);
}


PLSTATUS PLEXPORT
plsys_tsigmask(int how, const sigset_t *newset, sigset_t *oldset)
{
	sigset_t realset;

	if (how != SIG_BLOCK && how != SIG_UNBLOCK && how != SIG_SETMASK) {
		return PLSTATUS_INVALID;
	}

	/*
	 * SIGCANCEL and SIGSETXID should not be blocked.
	 */
	if (newset != NULL && (how == SIG_BLOCK || how == SIG_SETMASK)) {
		plsys_memmove(&realset, newset, sizeof(realset));

		plsys_sigdelset(&realset, SIGCANCEL);
		plsys_sigdelset(&realset, SIGSETXID);

		newset = &realset;
	}

	return plsys_tsigmask_nt(how, newset, oldset);
}


