/**
 * @file src/lib/plc/sysdep/linux/i386/plsys_twait.c
 */
#include <plc.h>


PLSTATUS PLEXPORT
plsys_twait(void *addr, long val, long timeout)
{
	struct timespec tv, *tvp;
	long err, e;

	if (addr == NULL) {
		return PLSTATUS_INVALID;
	}

	if (timeout == -1) {
		tvp = NULL;
	} else {
		tvp = &tv;
		plsys_ms2timespec(tvp, timeout);
	}

	e = _plsyscall(&err, SYS_futex, addr, FUTEX_WAIT, val, tvp, NULL, 0);
	if (e < 0) {
		errno = err;
		switch (err) {
		case ETIMEDOUT:
			return PLSTATUS_TIMEOUT;

		case EAGAIN:
		case EINTR:
			return PLSTATUS_TRY_AGAIN;

		default:
			return PLSTATUS_FAILED;
		}
	}

	return PLSTATUS_SUCCESS;
}


