/*
 * Test of peludo sockets using posix wrappers.
 */
#include <sys/types.h>
#include <sys/socket.h> 
#include <netinet/in.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

void test1();

int
main(int argc, char **argv)
{
	plprint("%s: Initializing PLC API...\n", __FUNCTION__);
	plinit();

	plprint("%s: Running test...\n", __FUNCTION__);
	test1();

	plprint("%s: Releasing PLC API...\n", __FUNCTION__);
	plrelease();

	return 0;
}

void
test1()
{
	int socket_desc;
	struct sockaddr_in address;
	socklen_t addrlen;
	int new_socket;
	char buf[1024];
	ssize_t len;

	socket_desc = socket(AF_INET, SOCK_STREAM, 0);
	if (socket_desc == 0) {
		perror("Can't create socket");
		exit(1);
	}

	printf("Socket created: %d\n", socket_desc);

	address.sin_family = AF_INET;
	address.sin_addr.s_addr = INADDR_ANY;
	address.sin_port = htons(1234);

	if (bind(socket_desc, (struct sockaddr *)&address, sizeof(address))
	    < 0) {
		perror("Can't bind");
		exit(1);
	}

	printf("Bind success\n");

	if (listen(socket_desc, 3) < 0) {
		perror("Can't listen");
		exit(1);
	}

	printf("Listen success\n");

	addrlen = sizeof(address);
	new_socket = accept(socket_desc, (struct sockaddr *)&address, &addrlen);
	if (new_socket < 0) {
		perror("Can't accept");
		exit(1);
	}

	printf("New socket is %d\n", new_socket);

#define WELCOME	"Peludo socket test1: Echo server started\n" \
		"Hi, write 'quit' to finish.\n"

	send(new_socket, WELCOME, strlen(WELCOME), 0);

	while (1) {
		len = recv(new_socket, buf, sizeof(buf), 0);
		if (len <= 0)  {
			perror("recv failed");
			break;
		}

		buf[len] = '\0';

		send(new_socket, buf, len, 0);

		if (!strncmp(buf, "quit\n", len)) {
			break;
		}
	}

	close(new_socket);
	close(socket_desc);
}


