/*
 * PLT API tests: Conditions
 */
#include <plc.h>
#include <pllog.h>

#include <pthread.h>


volatile long threads = 0;
volatile long end = 0;


void test1();

int
main(int argc, char **argv)
{
	pldebug("%s: Initializing PLC API...\n", __FUNCTION__);
	plinit();

	pldebug("%s: Initializing PTHREAD API...\n", __FUNCTION__);
	pthread_init_np();

	pldebug("%s: Starting test...\n\n", __FUNCTION__);
	test1();

	pldebug("%s: Releasing PTHREAD API...\n", __FUNCTION__);
	pthread_release_np();

	pldebug("%s: Releasing PLC API...\n", __FUNCTION__);
	plrelease();

	return 0;
}


/*************************************************************
 * test-1:	SYNCHRONIZATION USING CONDITIONS
 *************************************************************/
static pthread_mutex_t mutex;
static pthread_cond_t cond;

void *
test1_thread(void *arg)
{
	long number = (long)arg;
	int err;

	//platomic_inc((long *)&threads);
	threads++;

//	while (!platomic_read((long *)&end)) {
	while (!end) {
		pldebug("Thread-%d: Waiting for the mutex...\n", number);
		pthread_mutex_lock(&mutex);

		pldebug("Thread-%d: Mutex owned. "
		    "Waiting for the condition...\n", number);
		err = pthread_cond_wait(&cond, &mutex);

		pldebug("Thread-%d: Woke up, error=<%d>\n", number, err);

		pthread_mutex_unlock(&mutex);
	}
	
	pldebug("Thread-%d: Finished\n", number);

//	platomic_dec((long *)&threads);
	threads--;

	return NULL;
}

void
test1()
{
	int i, err;

	pldebug("%s: Initializing mutex <%x>...\n", __FUNCTION__, &mutex);
	pthread_mutex_init(&mutex, NULL);

	pldebug("%s: Initializing condition <%x>...\n", __FUNCTION__, &cond);
	pthread_cond_init(&cond, NULL);

	pldebug("%s: Owning mutex until all threads are created...\n",
	    __FUNCTION__);

	if (pthread_mutex_lock(&mutex) != 0) {
		pldebug("%s: Can't own the mutex. Aborted.\n", __FUNCTION__);
		plsys_exit(-1);
		/* NOTREACHED */
	}

	for (i = 1; i <= 10; i++) {

		pldebug("%s: Creating thread %d...\n", __FUNCTION__, i);

		err = pthread_create(NULL, NULL, test1_thread, (void *)i);
		if (err != 0) {
			pldebug("%s: Can't plt_create: error=%d. Aborted.\n",
			    __FUNCTION__, err);

			plsys_exit(-1);
			/* NOTREACHED */
		}
	}

//	while (platomic_read((long *)&threads) != 10) {
	while (threads != 10) {
		plsys_msleep(100);
	}
	plsys_msleep(100);

	pldebug("%s: Unlocking mutex...\n", __FUNCTION__);
	pthread_mutex_unlock(&mutex);

	pldebug("%s: Sleeping for 2 seconds...\n", __FUNCTION__);
	plsys_msleep(2000);

	/*
	 * TEST 1 - SIGNAL 1 THREAD WAITING ON THE CONDITION
	 */
	pldebug("%s: Signaling condition, only one should wakeup...\n",
	    __FUNCTION__);
	pthread_cond_signal(&cond);
	plsys_msleep(1000);

	/*
	 * TEST 2 - SIGNAL ALL THREADS WAITING ON THE CONDITION
	 */
	pldebug("%s: Signaling condition, all should wakeup...\n",
	    __FUNCTION__);
	pthread_cond_broadcast(&cond);
	plsys_msleep(1000);

	/*
	 * END
	 */
//	platomic_inc((long *)&end);
	end++;

//	while (platomic_read((long *)&threads)) {
	while (threads) {
		pldebug("%s: Still waiting for %d threads to end...\n",
		    __FUNCTION__, threads);

		//same as pthread_cond_signal(&cond);
		pthread_cond_signal_np(&cond, 1, NULL);

		plsys_msleep(1000);
	}

	pldebug("%s: All threads are dead. Test finished.\n", __FUNCTION__);
	plsys_msleep(1000);
}


