/*
 * Peludo Threads API tests.
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <pthread.h>


void test1();
void test2();
void test3();


int
main(int argc, char **argv)
{
	plinit();
	pthread_init_np();

	test1();
	test2();
	test3();

	printf("\n"
	    "/***********************************************************/\n"
	    "/***********************************************************/\n");

#if 1
	exit(0);
#else
	pldebug("%s: Releasing PTHREAD API...\n", __FUNCTION__);

	pthread_release_np();

	printf("%s: Releasing PLC API...\n", __FUNCTION__);
	plrelease();
#endif

	return 0;
}


/*************************************************************
 * test-1:	SYNCHRONIZATION USING MUTEXES
 *************************************************************/
struct _test1 {
	pthread_mutex_t mutex;
	int	 newmsg;
	char	 buffer[256];
	pthread_mutex_t end;
};

void *
test1_echo_client(void *arg)
{
	struct _test1 *msg;
	int i;

	pldebug("%s: Started...\n", __FUNCTION__);

	msg = (struct _test1 *)arg;

	for (i = 0; i < 10; i++) {

		while (1) {
			pthread_mutex_lock(&msg->mutex);

			if (!msg->newmsg) {

				sprintf(msg->buffer, "Hello world %d", i);

				pldebug("%s: Sending <%s>\n", __FUNCTION__,
				    msg->buffer);

				msg->newmsg = 1;

				pthread_mutex_unlock(&msg->mutex);

				break;
			}

			pthread_mutex_unlock(&msg->mutex);

			/* The server is slow :) */
			plsys_msleep(10);
		}
	}

	/* Tell the server that the client is exiting */
	pthread_mutex_lock(&msg->mutex);
	msg->newmsg = -1;
	pthread_mutex_unlock(&msg->mutex);

	pldebug("%s: Calling pthread_exit()\n", __FUNCTION__);
	
	//ptread_exit((void *)1234);
	exit(1234);
	/* NOTREACHED */

	/* Avoid gcc warning */
	return NULL;
}

void *
test1_echo_server(void *arg)
{
	struct _test1 *msg;

	pldebug("%s: Started\n", __FUNCTION__);

	msg = (struct _test1 *)arg;

	/* Own end mutex until the server ends */
	pthread_mutex_lock(&msg->end);

	while (1) {
		pthread_mutex_lock(&msg->mutex);

		if (msg->newmsg == 1) {

			pldebug("%s: Received <%s>\n", __FUNCTION__,
			    msg->buffer);

			msg->newmsg = 0;

		} else if (msg->newmsg == -1) {
			break;
		}

		pthread_mutex_unlock(&msg->mutex);
	}

	pldebug("%s: Leaving\n", __FUNCTION__);

	/* Tell test1 that the server is exiting */
	pthread_mutex_unlock(&msg->end);

	/* Return (the PLT API should call pthread_exit() for us) */
	return ((void *)4321);
}

void
test1()
{
	pthread_t echo_client, echo_server;
	struct _test1 arg;
	pthread_attr_t attr;
	int err;

	printf("\n"
	    "/***********************************************************\n"
	    " * test-1:	SYNCHRONIZATION USING MUTEXES\n"
	    " ***********************************************************/\n");

	bzero((void *)&arg, sizeof(arg));

	pldebug("%s: Initializing attributes...\n", __FUNCTION__);
	pthread_attr_init(&attr);

	pldebug("%s: Initializing mutexes...\n", __FUNCTION__);
	pthread_mutex_init(&arg.mutex, NULL);
	pthread_mutex_init(&arg.end, NULL);

	pldebug("%s: Creating echo server...\n", __FUNCTION__);
	err = pthread_create(&echo_server, NULL, test1_echo_server, (void *)&arg);
	if (err != 0) {
		pldebug("%s: Can't ptread_create server: error=%d\n",
		    __FUNCTION__, err);
		return;
	}
	pldebug("%s: echo server created at %x...\n",
	    __FUNCTION__, echo_server);

	pldebug("%s: Creating echo client...\n", __FUNCTION__);
	err = pthread_create(&echo_client, &attr, test1_echo_client, (void *)&arg);
	if (err != 0) {
		pldebug("%s: Can't pthread_create client: error=%d\n",
		    __FUNCTION__, err);
		return;
	}
	pldebug("%s: echo client created at %x...\n",
	    __FUNCTION__, echo_client);

	/*
	 * "Be sure" the server got the end mutex
	 */
	plsys_msleep(1000);

	/*
	 * Wait for the end mutex before extiting
	 */
	pthread_mutex_lock(&arg.end);
}


/*************************************************************
 * test-2:	LOW LEVEL WAIT ON AN ADDRESS
 *************************************************************/
volatile long __attribute__((aligned(sizeof(long)))) fut = 1;

void *
test2_1(void *arg)
{
	PLSTATUS ret;

	pldebug("%s is running\n", __FUNCTION__);

	while ((ret = plsys_twait((void *)&fut, 0, -1)) == PLSTATUS_TRY_AGAIN) {
		pldebug("%s: Trying again...\n", __FUNCTION__);
		plsys_msleep(100);
	}

	pldebug("%s: Returned because=<%s>, errno=%d\n",
	    __FUNCTION__,
	    plstrstatus(ret),
	    errno);

	plsys_msleep(1000);
	return NULL;
}

void *
test2_2(void *arg)
{
	PLSTATUS st;

	pldebug("%s is running\n", __FUNCTION__);

	while ((st = plsys_twait((void *)&fut, 1, -1)) == PLSTATUS_TRY_AGAIN) {
		pldebug("%s: Trying again...\n", __FUNCTION__);
		plsys_msleep(100);
	}

	pldebug("%s, returned because=<%s>, errno=%d\n", __FUNCTION__,
	    plstrstatus(st), errno);

	plsys_msleep(1000);
	return NULL;
}

void *
test2_3(void *arg)
{
	long ret;

	pldebug("%s is running\n", __FUNCTION__);

	plsys_msleep(1000);
	fut = 0;
	do {
		ret = plsys_tsignal((void *)&fut, 1);
		pldebug("%s. tsignal returned value=%d\n", __FUNCTION__, ret);
	} while (ret <= 0);

	pldebug("%s, woken up=%d, errno=%d\n", __FUNCTION__, ret, errno);

	plsys_msleep(1000);
	return NULL;
}

void
test2()
{
	printf("\n"
	    "/***********************************************************\n"
	    " * test-2:	LOW LEVEL WAIT ON AN ADDRESS\n"
	    " ***********************************************************/\n");

	(void)pthread_create(NULL, NULL, test2_1, NULL);
	(void)pthread_create(NULL, NULL, test2_2, NULL);
	(void)pthread_create(NULL, NULL, test2_3, NULL);

	plsys_msleep(3000);
}


/*************************************************************
 * test-3:	SYNCHRONIZATION USING CONDITIONS
 *************************************************************/
struct _test3 {
	pthread_cond_t cond;
	pthread_mutex_t mutex;

	int	 newmsg;
	char	 buffer[256];

	pthread_cond_t endcond;
	pthread_mutex_t end;
};

void *
test3_echo_client(void *arg)
{
	struct _test3 *msg;
	int i, err;

	pldebug("%s: Started...\n", __FUNCTION__);

	msg = (struct _test3 *)arg;

	for (i = 1; i <= 10; i++) {

		while (1) {
			/*
			 * Own the message structure lock
			 */
			pthread_mutex_lock(&msg->mutex);

			/*
			 * If the server got the previous message
			 * send a new one, otherwise sleep for 100ms
			 * and try again.
			 */
			if (!msg->newmsg) {
				/*
				 * Create the new message
				 */
				sprintf(msg->buffer, "Hello world %d", i);

				pldebug("%s: Sending <%s>\n",
				    __FUNCTION__,
				    msg->buffer);

				/*
				 * Tell the server there is a new message
				 */
				msg->newmsg = 1;

				pthread_mutex_unlock(&msg->mutex);

				err = pthread_cond_signal(&msg->cond);
				if (err != 0) {
					pldebug("%s: Signal of message "
					    "condition returned error=%d. "
					    "ABORT\n",
					    __FUNCTION__,
					    err);

					plsys_exit(-1);
					/* NOTREACEHD */
				}

				/*
				 * Prepare to send next message
				 */
				break;
			}

			/*
			 * The server is not ready:
			 * Release the message structure lock and try again.
			 */
			pthread_mutex_unlock(&msg->mutex);

			/* The server is slow :) */
			plsys_msleep(100);
		}
	}

	/*
	 * Tell the server client is exiting
	 */
	pthread_mutex_lock(&msg->mutex);
	msg->newmsg = -1;
	pthread_mutex_unlock(&msg->mutex);
	pthread_cond_signal(&msg->cond);

	pldebug("%s: Calling pthread_exit()\n", __FUNCTION__);
	
	pthread_exit((void *)1234);
	/* NOTREACHED */

	/* Avoid gcc warning */
	return NULL;
}

void *
test3_echo_server(void *arg)
{
	struct _test3 *msg;
	int err;

	pldebug("%s: Started\n", __FUNCTION__);

	msg = (struct _test3 *)arg;

	/*
	 * Lock the message mutex.
	 */
	pthread_mutex_lock(&msg->mutex);

	while (1) {
		/*
 		 * Check for a new message from the client.
 		 */
		if (msg->newmsg == 1) {

			pldebug("%s: Received <%s>\n",
			    __FUNCTION__,
			    msg->buffer);

			msg->newmsg = 0;

		} else if (msg->newmsg == -1) {
			/*
			 * Client exit, so the server.
			 */
			pthread_mutex_unlock(&msg->mutex);
			break;
		}

		/*
 		 * Sleep on the message condition until the client signals it.
 		 * (plt_cond_wait() will unlock the mutex before going to
 		 * sleep and re-own it after a wakeup)
		 */
		err = pthread_cond_wait(&msg->cond, &msg->mutex);
		pldebug("%s: Wait on message condition returned error=<%d>\n",
		    __FUNCTION__,
		    err);
	}

	pldebug("%s: Leaving\n", __FUNCTION__);

	/*
	 * Tell the main thread the server is exiting
	 */
	pthread_cond_broadcast(&msg->endcond);

	/*
	 * Return (the PTHREAD API should call pthread_exit() for us).
	 */
	return ((void *)4321);
}

void
test3()
{
	pthread_t echo_client, echo_server;
	struct _test3 arg;
	pthread_attr_t attr;
	int err;

	printf("\n"
	    "/***********************************************************\n"
	    " * test-3:	SYNCHRONIZATION USING CONDITIONS\n"
	    " ***********************************************************/\n");

	bzero((void *)&arg, sizeof(arg));

	pldebug("%s: Initializing attributes...\n", __FUNCTION__);
	pthread_attr_init(&attr);


	pldebug("%s: Initializing mutexes...\n", __FUNCTION__);
	pthread_mutex_init(&arg.mutex, NULL);
	pthread_mutex_init(&arg.end, NULL);


	pldebug("%s: Initializing conditions...\n", __FUNCTION__);
	pthread_cond_init(&arg.cond, NULL);
	pthread_cond_init(&arg.endcond, NULL);


	/*
	 * Own the ending mutex
	 */
	pthread_mutex_lock(&arg.end);


	/*
	 * Create server thread
	 */
	pldebug("%s: Creating echo server...\n", __FUNCTION__);
	err = pthread_create(&echo_server, NULL, test3_echo_server, (void *)&arg);
	if (err != 0) {
		pldebug("%s: Can't pthread_create server: %d\n",
		    __FUNCTION__, err);
		return;
	}
	pldebug("%s: echo server created at %x...\n\n",
	    __FUNCTION__, echo_server);


	/*
	 * Create client thread
	 */
	pldebug("%s: Creating echo client...\n", __FUNCTION__);
	err = pthread_create(&echo_client, &attr, test3_echo_client, (void *)&arg);
	if (err != 0) {
		pldebug("%s: Can't plt_create client: %d\n",
		    __FUNCTION__, err);
		return;
	}
	pldebug("%s: echo client created at %x...\n\n",
	    __FUNCTION__, echo_client);


	/*
	 * Wait for the end condition to be signaled
	 */
	pthread_cond_wait(&arg.endcond, &arg.end);


	/*
	 * Release conditions.
	 */
	pldebug("%s: Releasing condition endcond...\n", __FUNCTION__);
again1:	if ((err = pthread_cond_destroy(&arg.endcond)) != 0) {

		if (err == EBUSY) {
			plsys_msleep(100);
			goto again1;
		}

		pldebug("%s: Cant release endoncd: error=%d. ABORTED\n",
		    __FUNCTION__, err);

		plsys_exit(-1);
		/* NOTREACHED */
	}

	pldebug("%s: Releasing condition arg...\n", __FUNCTION__);
again2:	if ((err = pthread_cond_destroy(&arg.cond)) != 0) {

		if (err == EBUSY) {
			plsys_msleep(100);
			goto again2;
		}

		pldebug("%s: Cant release arg condition: error=%d. ABORTED\n",
		    __FUNCTION__, err);

		plsys_exit(-1);
		/* NOTREACHED */
	}

}


