
/* FreeBSD/i386 */
#define MMAP_ADDRESS			0x08048000
#define OS				ELF_OSABI_FREEBSD
/****************/


#define ELF_CLASS_32			0x01
#define ELF_CLASS_64			0x02

#define ELF_DATA_LE			0x01	/* Little endian	*/
#define ELF_DATA_BE			0x02	/* Big Endian		*/

#define ELF_TYPE_EXEC			0x0002	/* Executable File	*/
#define ELF_TYPE_SHARED			0x0003	/* Shared Object File	*/

#define ELF_MACHINE_NONE		0	/* Unknown machine	*/
#define ELF_MACHINE_M32			1	/* AT&T WE32100		*/
#define ELF_MACHINE_SPARC		2	/* Sun SPARC		*/
#define ELF_MACHINE_386			3	/* Intel i386		*/
#define ELF_MACHINE_68K			4	/* Motorola 68000	*/
#define ELF_MACHINE_88K			5	/* Motorola 88000	*/
#define ELF_MACHINE_860			7	/* Intel i860		*/
#define ELF_MACHINE_MIPS		8	/* MIPS R3000 Big-Endian only */
#define ELF_MACHINE_S370		9	/* IBM System/370	*/
#define ELF_MACHINE_MIPS_RS3_LE		10	/* MIPS R3000 Little-Endian */
#define ELF_MACHINE_PARISC		15	/* HP PA-RISC		*/
#define ELF_MACHINE_VPP500		17	/* Fujitsu VPP500	*/
#define ELF_MACHINE_SPARC32PLUS		18	/* SPARC v8plus		*/
#define ELF_MACHINE_960			19	/* Intel 80960		*/
#define ELF_MACHINE_PPC			20	/* PowerPC 32-bit	*/
#define ELF_MACHINE_PPC64		21	/* PowerPC 64-bit	*/
#define ELF_MACHINE_S390		22	/* IBM System/390	*/
#define ELF_MACHINE_V800		36	/* NEC V800		*/
#define ELF_MACHINE_FR20		37	/* Fujitsu FR20		*/
#define ELF_MACHINE_RH32		38	/* TRW RH-32		*/
#define ELF_MACHINE_RCE			39	/* Motorola RCE		*/
#define ELF_MACHINE_ARM			40	/* ARM			*/
#define ELF_MACHINE_SH			42	/* Hitachi SH		*/
#define ELF_MACHINE_SPARCV9		43	/* SPARC v9 64-bit	*/
#define ELF_MACHINE_TRICORE		44	/* Siemens TriCore embedded */
#define ELF_MACHINE_ARC			45	/* Argonaut RISC Core	*/
#define ELF_MACHINE_H8_300		46	/* Hitachi H8/300	*/
#define ELF_MACHINE_H8_300H		47	/* Hitachi H8/300H	*/
#define ELF_MACHINE_H8S			48	/* Hitachi H8S		*/
#define ELF_MACHINE_H8_500		49	/* Hitachi H8/500	*/
#define ELF_MACHINE_IA_64		50	/* Intel IA-64		*/
#define ELF_MACHINE_MIPS_X		51	/* Stanford MIPS-X	*/
#define ELF_MACHINE_COLDFIRE		52	/* Motorola ColdFire	*/
#define ELF_MACHINE_68HC12		53	/* Motorola M68HC12	*/
#define ELF_MACHINE_MMA			54	/* Fujitsu MMA		*/
#define ELF_MACHINE_PCP			55	/* Siemens PCP		*/
#define ELF_MACHINE_NCPU		56	/* Sony nCPU		*/
#define ELF_MACHINE_NDR1		57	/* Denso NDR1 		*/
#define ELF_MACHINE_STARCORE		58	/* Motorola Star*Core	*/
#define ELF_MACHINE_ME16		59	/* Toyota ME16 processor */
#define ELF_MACHINE_ST100		60	/* STMicroelectronics ST100 */
#define ELF_MACHINE_TINYJ		61	/* Advanced Logic Corp. TinyJ */
#define ELF_MACHINE_AMD64		62	/* AMD x86-64		*/

#define ELF_OSABI_NONE			0x00	/* UNIX System V ABI	*/
#define ELF_OSABI_HPUX			0x01	/* HP-UX operating system */
#define ELF_OSABI_NETBSD		0x02	/* NetBSD		*/
#define ELF_OSABI_LINUX			0x03	/* Linux		*/
#define ELF_OSABI_HURD			0x04	/* Hurd			*/
#define ELF_OSABI_86OPEN		0x05	/* 86Open common IA32 ABI */
#define ELF_OSABI_SOLARIS		0x06	/* Solaris		*/
#define ELF_OSABI_AIX			0x07	/* AIX			*/
#define ELF_OSABI_IRIX			0x08	/* IRIX			*/
#define ELF_OSABI_FREEBSD		0x09	/* FreeBSD		*/
#define ELF_OSABI_TRU64			0x0A	/* TRU64 UNIX		*/
#define ELF_OSABI_MODESTO		0x0B	/* Novell Modesto	*/
#define ELF_OSABI_OPENBSD		0x0C	/* OpenBSD		*/
#define ELF_OSABI_OPENVMS		0x0D	/* Open VMS		*/
#define ELF_OSABI_NSK			0x0E	/* HP Non-Stop Kernel	*/
#define ELF_OSABI_ARM			0x61	/* ARM			*/
#define ELF_OSABI_STANDALONE		0xFF	/* Standalone (embedded) Appl */

#define ELF_VERSION_CURRENT		1


.text

elf_hdr:
	/* IDENT - MAGIC */
	.byte	0x7F, 'E', 'L', 'F'

	/* IDENT - CLASS */
	.byte	ELF_CLASS_32

	/* IDENT - DATA ENCODING */
	.byte	ELF_DATA_LE

	/* IDENT - VERSION */
	.byte	ELF_VERSION_CURRENT

	/* IDENT - OS ABI */
	.byte	OS

	/* IDENT - ABI VERSION */
	.byte	0x00

	/* IDENT - PAD */
	.byte	0x00, 0x00, 0x00, 0x00, 0x00

	/* IDENT - BRAND */
	.byte	0x00

	/* IDENT - SIZE OF IDENT */
	.byte	0x10

	/* TYPE OF FILE */
	.short	ELF_TYPE_EXEC

	/* MACHINE */
	.short	ELF_MACHINE_386

	/* VERSION */
	.int	ELF_VERSION_CURRENT

	/* ENTRY POINT */
	.int	MMAP_ADDRESS + _start - elf_hdr

	/* Offset to the Program Header Table */
	.int	(program_hdr - elf_hdr)

	/* Offset to the Section Header Table */
	.int	(section_hdr - elf_hdr)

	/* Processor Flags */
	.int	0x00000000

	/* ELF Header Size */
	.short	(elf_hdr_end - elf_hdr)

	/* Size of an Entry in the Program Header Table */
	.short	(program_hdr_end - program_hdr)

	/* Number of Entries in the Program Header Table */
	.short	0x0001

	/* Size of an Entry in the Section Header Table */
	.short	(section_hdr_end - section_hdr)

	/* Numbe of Entries in the Section Header Table */
	.short	0x0001

	/* Section Header String Index */
	.short	0x0000
elf_hdr_end:

program_hdr:
	/* Type: 1 = PT_LOAD: Load a Segment */
	.int	0x00000001

	/* Offset to the segment to load (from the begining of the file) */
	.int	0x00000000

	/* Virtual Address for the Segment */
	.int	MMAP_ADDRESS

	/* Physical Address for the Segment */
	.int	MMAP_ADDRESS

	/*
	 * File Size:
	 * THE SCRIPT SHOULD SET THIS VALUE
	 * OFFSET: 0x44
	 */
	.int	0x00000000

	/*
	 * Image Size:
	 * SCRIPT SHOULD SET THIS VALUE
	 * OFFSET: 0x48
	 */
	.int	0x00000000

	/* Segment Flags: R=4, W=2, X=1 */
	.int	0x00000005

	/* Segment Memory Alignment */
	.int	0x00001000
program_hdr_end:

section_hdr:
	/*
	 * SECTION .TEXT
	 */
	/* Section Name (index inside the string table) */
	.int	0x00000000

	/* Section Type: PROGBITS */
	.int	0x00000001

	/* Section Flags: ALLOC=2, EXECUTABLE_INSTRUCTIONS=4 */
	.int	0x00000006

	/* Section Address */
	.int	MMAP_ADDRESS

	/* Section Offset */
	.int	MMAP_ADDRESS

	/*
	 * Section Size
	 * THE SCRIPT SHOULD ADD THE PLB SIZE TO THIS VALUE
	 * OFFSET: 0x68
	 */
	.int	plb_start - _start

	/* Section Link */
	.int	0x00000000

	/* Section Info */
	.int	0x00000000

	/* Section Address Align */
	.int	0x00001000

	/* Section Ent Size */
	.int	0x00000000
section_hdr_end:

/*
 * _start(argc, arg1, arg2, ..., 0, env1, env2, ..., 0)
 */
_start:		popl	%eax
		pushl	%esp
		pushl	%eax
		call	plb_start
		push	%eax
		push	$0
		movl	$1, %eax	/* SYS_exit */
		int	$0x80
		hlt


/*
 * The PLB should be located at the beginning of a page
 */
.align 0x1000, 0
plb_start:

